package com.moyosoft.samples.excel.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

import javax.swing.*;

import com.moyosoft.connector.com.*;
import com.moyosoft.connector.exception.*;
import com.moyosoft.connector.ms.excel.*;
import com.moyosoft.samples.excel.*;

public class ExcelDemo extends JFrame
{
    private Excel excelApplication;
    
    public ExcelDemo()
    {
        super("Java Excel Connector - Sample Demo Applications");
        init();
    }

    protected void init()
    {
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent e)
            {
                exitApplication();
            };

            public void windowOpened(WindowEvent e)
            {
                initApplication();
            }
        });

        ExcelSampleAppPanel helloExcelPanel = new ExcelSampleAppPanel(
                "HelloExcel",
                "<html>This is a minimal sample application. The application connects to Excel and displays<br>the Excel version in the console.</html>"
                );
        
        helloExcelPanel.addButtonRunActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performRunHelloExcel();
            }
        });

        helloExcelPanel.addButtonViewSourceActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performViewSourceHelloExcel();
            }
        });

        ExcelSampleAppPanel createWorkbookPanel = new ExcelSampleAppPanel(
                "CreateWorkbook",
                "This sample application creates a new Excel workbook and fills the cells with data.");
        
        createWorkbookPanel.addButtonRunActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performRunCreateWorkbook();
            }
        });

        createWorkbookPanel.addButtonViewSourceActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performViewSourceCreateWorkbook();
            }
        });
        
        ExcelSampleAppPanel openWorkbookPanel = new ExcelSampleAppPanel(
                "OpenWorkbook",
                "This sample application shows how to open an existing Excel document.");

        openWorkbookPanel.addButtonRunActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performRunOpenWorkbook();
            }
        });

        openWorkbookPanel.addButtonViewSourceActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performViewSourceOpenWorkbook();
            }
        });
        
        ExcelSampleAppPanel createChartPanel = new ExcelSampleAppPanel(
                "CreateChart",
                "This sample application creates a new chart displaying random data.");

        createChartPanel.addButtonRunActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performRunCreateChart();
            }
        });

        createChartPanel.addButtonViewSourceActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                performViewSourceCreateChart();
            }
        });
        
        JPanel mainPanel = new JPanel(new BorderLayout());

        mainPanel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        Insets insets = new Insets(8, 16, 0, 16);
        Insets separatorInsets = new Insets(4, 16, 0, 16);

        c.fill = GridBagConstraints.BOTH;
        c.anchor = GridBagConstraints.NORTHWEST;
        c.gridx = 0;

        c.gridy = 0;
        c.insets = insets;
        mainPanel.add(helloExcelPanel, c);

        c.gridy = 1;
        c.insets = separatorInsets;
        mainPanel.add(new EtchedLine(), c);
        
        c.gridy = 2;
        c.insets = insets;
        mainPanel.add(createWorkbookPanel, c);

        c.gridy = 3;
        c.insets = separatorInsets;
        mainPanel.add(new EtchedLine(), c);
        
        c.gridy = 4;
        c.insets = insets;
        mainPanel.add(openWorkbookPanel, c);

        c.gridy = 5;
        c.insets = separatorInsets;
        mainPanel.add(new EtchedLine(), c);
        
        c.gridy = 6;
        c.insets = insets;
        mainPanel.add(createChartPanel, c);

        JPanel panel1 = new JPanel(new BorderLayout());
        panel1.add(mainPanel, BorderLayout.WEST);
        
        JPanel panel2 = new JPanel(new BorderLayout());
        panel2.add(panel1, BorderLayout.NORTH);
        
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(panel2, BorderLayout.CENTER);

        setSize(480, 490);
        centerOnScreen(this);
    }
    
    private URL loadSourceCodeFile(String filename)
    {
        return ExcelDemo.class.getClassLoader().getResource("html/" + filename);
    }
    
    private void performRunHelloExcel()
    {
        PrintStream standardSystemOut = System.out;
        
        StringBufferOutputStream output = new StringBufferOutputStream();
        System.setOut(new PrintStream(output));
        
        HelloExcel.main(null);
        
        JOptionPane.showMessageDialog(this, output.toString());
        
        System.setOut(standardSystemOut);
    }
    
    private void performViewSourceHelloExcel()
    {
        SourceCodeDialog.open(this, loadSourceCodeFile("hello_excel.html"));
    }
    
    private void performRunCreateWorkbook()
    {
        CreateWorkbook.main(null);
    }
    
    private void performViewSourceCreateWorkbook()
    {
        SourceCodeDialog.open(this, loadSourceCodeFile("create_workbook.html"));
    }
    
    private void performRunOpenWorkbook()
    {
        OpenWorkbook.main(null);
    }
    
    private void performViewSourceOpenWorkbook()
    {
        SourceCodeDialog.open(this, loadSourceCodeFile("open_workbook.html"));
    }
    
    private void performRunCreateChart()
    {
        CreateChart.main(null);
    }
    
    private void performViewSourceCreateChart()
    {
        SourceCodeDialog.open(this, loadSourceCodeFile("create_chart.html"));
    }
    
    private static void centerOnScreen(Window window)
    {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension windowSize = window.getSize();
        windowSize.height = Math.min(windowSize.height + 20, screenSize.height);
        windowSize.width = Math.min(windowSize.width, screenSize.width);
        window.setLocation((screenSize.width - windowSize.width) / 2,
                (screenSize.height - windowSize.height) / 2);
    }

    protected void initApplication()
    {
        try
        {
            excelApplication = new Excel();
        }
        catch(LibraryNotFoundException e)
        {
            JOptionPane.showMessageDialog(this, "Unable to find the library.");
            e.printStackTrace();
            System.exit(0);
        }
        catch(ComponentObjectModelException e)
        {
            ComErrorDialog.open(this, e);
        }
    }

    private void exitApplication()
    {
        if(excelApplication != null)
        {
            excelApplication.dispose();
            excelApplication = null;
        }

        dispose();
        //System.exit(0);
    }

    public static void open()
    {
        ExcelDemo frame = new ExcelDemo();
        frame.show();
    }

    public static void main(String[] args)
    {
        // Set the look and feel
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch(Exception e)
        {
        }
        
        ExcelDemo.open();
    }
}

class StringBufferOutputStream extends OutputStream
{
    private StringBuffer buf = new StringBuffer();
    
    public void write(int b) throws IOException
    {
        buf.append(String.valueOf((char)b));
    }
    
    public String toString()
    {
        return buf.toString();
    }
}

class ExcelSampleAppPanel extends JPanel
{
    public static final Icon BULLET = loadIcon("imgs/bullet.gif");

    private static Icon loadIcon(String pIconPath)
    {
      try
      {
        return new ImageIcon(ExcelSampleAppPanel.class.getClassLoader().getResource(pIconPath));
      }
      catch(Exception ex)
      {
        return null;
      }
    }

    private String title;
    private String description;
    
    private JButton buttonRun = new JButton("Run application");
    private JButton buttonViewSource = new JButton("View source code");

    public ExcelSampleAppPanel(String title, String description)
    {
        this.title = title;
        this.description = description;
        init();
    }

    public void addButtonRunActionListener(ActionListener actionListener)
    {
        buttonRun.addActionListener(actionListener);
    }
    
    public void addButtonViewSourceActionListener(ActionListener actionListener)
    {
        buttonViewSource.addActionListener(actionListener);
    }
    
    protected void init()
    {
        JPanel buttonsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        buttonsPanel.add(buttonRun);
        buttonsPanel.add(buttonViewSource);

        JLabel titleLabel = new JLabel(title);
        titleLabel.setFont(new Font(getFont().getName(), Font.BOLD, 15));
        titleLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 8, 0));
        
        JLabel descriptionLabel = new JLabel(description);
        descriptionLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 8, 0));
        
        JPanel mainPanel = new JPanel(new BorderLayout());
        mainPanel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
        mainPanel.add(titleLabel, BorderLayout.NORTH);
        mainPanel.add(descriptionLabel, BorderLayout.CENTER);
        mainPanel.add(buttonsPanel, BorderLayout.SOUTH);

        JPanel imagePanel = new JPanel(new BorderLayout());
        imagePanel.add(new JLabel(BULLET), BorderLayout.NORTH);
        imagePanel.setBorder(BorderFactory.createEmptyBorder(9, 5, 0, 6));
        
        this.setLayout(new BorderLayout());
        this.add(mainPanel, BorderLayout.CENTER);
        this.add(imagePanel, BorderLayout.WEST);
    }
}
